#ifndef HARKIO_HARKCONFIG_HARKCONFIGFILEWRITER_HPP_
#define HARKIO_HARKCONFIG_HARKCONFIGFILEWRITER_HPP_

#include <boost/shared_ptr.hpp>
#include <harkio2/InvalidHarkFile.hpp>

/** @file HarkConfigFileWriter.hpp
 * @brief HarkConfigFileWriter クラスの定義
 */

namespace harkio2
{
namespace harkconfig
{

class HarkConfigFile;
class ImpulseResponseListFile1_0;
class ImpulseResponseListFile2_0;
class TSPListFile1_0;
class TSPListFile2_0;
class MicArrayLocationFile2_0;
class NoiseLocationFile2_0;

/** @brief HARK-configファイルの書き込みインターフェース。
 *
 * このインターフェースを実装した、各種ファイル形式（例えばXML形式）
 * に対応したクラスを用いることにより、様々なフォーマットのHARK-configファイルへデータを書き出すことができる。
 *
 * @sa HarkConfigFileXMLWriter
 */
class HarkConfigFileWriter
{
public:
    /// @brief 継承のための仮想デストラクタ
    virtual ~HarkConfigFileWriter()
    {}

    /** @brief HARK-configファイルへ指定されたデータを書き出す。
     *
     * HARK-configファイルのデータをどこかへ書き出す。
     * 書き出す場所はサブクラスのコンストラクタやメソッドにより指定されるだろう。
     *
     * このインターフェースでは、HARK-configファイルが本当のファイルであるかどうかは規定していないため、
     * 例えば名前付きパイプなどへ書き出すようなサブクラスを定義することもできる。
     * @param data 書き出すデータ
     */
    virtual void Write(boost::shared_ptr<harkio2::harkconfig::HarkConfigFile> data) throw (harkio2::InvalidHarkFile) = 0;

    /*
     * 各種データ構造用の書き出しメソッド
     * データ構造を新設するたびに追加する必要がある
     */
    virtual void Write(const harkio2::harkconfig::HarkConfigFile& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::ImpulseResponseListFile1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::ImpulseResponseListFile2_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::TSPListFile1_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::TSPListFile2_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::MicArrayLocationFile2_0& o) throw (harkio2::InvalidHarkFile) = 0;
    virtual void Write(const harkio2::harkconfig::NoiseLocationFile2_0& o) throw (harkio2::InvalidHarkFile) = 0;
};

}
}

#endif // HARKIO_HARKCONFIG_HARKCONFIGFILEWRITER_HPP_

