#ifndef HARKIO_HARKCONFIG_NOISESOURCE_HPP_
#define HARKIO_HARKCONFIG_NOISESOURCE_HPP_

#include <harkio2/defineitem.hpp>
#include <harkio2/Position.hpp>
#include <glibmm/ustring.h>

/** @file NoiseSource.hpp
 * @brief NoiseSource class and subclasses definitions.
 */

namespace harkio2
{
namespace harkconfig
{

/** @brief A noise information.
 * This class specifies information about a noise of a noise array.
 */
class NoiseSource
{
public:
    virtual ~NoiseSource() {}

    /** @brief Returns the data structure version.
     * Returns the version which this instance actually represents.
     * @return The version of this structure.
     */
    virtual unsigned int GetVersion() const = 0;
};

/** @brief A sound source information version 1.
 */
class NoiseSource1 : public NoiseSource
{
public:
    /** @brief Returns the noise version.
     * @return The version which this class represents.
     */
    virtual unsigned int GetVersion() const { return 1; }

    /// @brief The index number of this noise.
    DEFINE_ITEM(noiseIndex, unsigned int, NoiseIndex, unsigned int)

    /// @brief The position or direction of this noise.
    DEFINE_ITEM(position, harkio2::Position<float>, Position, const harkio2::Position<float>&)

    /// @brief A comment.
    DEFINE_ITEM(comment, Glib::ustring, Comment, const Glib::ustring&)

};

}
}

#endif // HARKIO_HARKCONFIG_NOISESOURCE_HPP_

