#ifndef HARKIO_HARKCONFIG_NODEPARSERS_HPP_
#define HARKIO_HARKCONFIG_NODEPARSERS_HPP_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <libxml++/parsers/textreader.h>
#include <harkio2/FileType.hpp>

/** @file nodeparsers.hpp
 * @brief XMLのノードパーサーの定義群。
 *
 * このファイルには ParseXXX という名前の関数が定義されている。
 * ParseXXX 関数はXMLの"XXX"タグをパースして返す。
 */

namespace harkio2
{

template <typename T> class Position;
class TypedFilename;

namespace harkconfig
{

class Source2;
class File1;
class Mic1;
class NoiseSource1;
class ImpulseResponseListFile;
class ImpulseResponseListFile2_0;
class TSPListFile;
class TSPListFile2_0;
class MicArrayLocationFile;
class MicArrayLocationFile2_0;
class NoiseLocationFile;
class NoiseLocationFile2_0;

/** @brief Converts string file type to FileType::Kind.
 * @param src[in] The string which represents a file type.
 * @param dst[out] The file type.
 * @return true if conversion succeeded.
 */
bool ConvertFileType(const Glib::ustring& src, harkio2::FileType::Kind& dst);

/*
 * parsers for various tags
 */
harkio2::Position<float>
    ParsePosition(xmlpp::TextReader& reader);
std::vector<unsigned int>
    ParseMicIndices(xmlpp::TextReader& reader);
harkconfig::File1
    ParseFile1(xmlpp::TextReader& reader);
std::vector<harkconfig::File1>
    ParseFiles1(xmlpp::TextReader& reader);
harkconfig::Source2
    ParseSource2(xmlpp::TextReader& reader);
std::vector<harkconfig::Source2>
    ParseSources2(xmlpp::TextReader& reader);
unsigned int
    ParseNumMic(xmlpp::TextReader& reader);
harkio2::FileType::Kind
    ParseSourceFileType(xmlpp::TextReader& reader);
Glib::ustring
    ParseComment(xmlpp::TextReader& reader);
unsigned int
    ParseSynchronousAverage(xmlpp::TextReader& reader);
unsigned int
    ParseTSPOffset(xmlpp::TextReader& reader);
unsigned int
    ParseTSPLength(xmlpp::TextReader& reader);
unsigned int
    ParseSignalMax(xmlpp::TextReader& reader);
harkio2::TypedFilename
    ParseOriginalImpulseFile(xmlpp::TextReader& reader);
harkconfig::Mic1
    ParseMic1(xmlpp::TextReader& reader);
std::vector<harkconfig::Mic1>
    ParseMics1(xmlpp::TextReader& reader);
harkconfig::NoiseSource1
    ParseNoiseSource1(xmlpp::TextReader& reader);
std::vector<harkconfig::NoiseSource1>
    ParseNoiseSources1(xmlpp::TextReader& reader);

/*
 * parsers for hark_config tag
 */
boost::shared_ptr<harkconfig::ImpulseResponseListFile> ParseImpulseResponseListFile(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::ImpulseResponseListFile2_0> ParseImpulseResponseListFile2_0(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::TSPListFile> ParseTSPListFile(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::TSPListFile2_0> ParseTSPListFile2_0(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::MicArrayLocationFile> ParseMicArrayLocationFile(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::MicArrayLocationFile2_0> ParseMicArrayLocationFile2_0(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::NoiseLocationFile> ParseNoiseLocationFile(xmlpp::TextReader& reader);

boost::shared_ptr<harkconfig::NoiseLocationFile2_0> ParseNoiseLocationFile2_0(xmlpp::TextReader& reader);

} // harkconfig
} // harkio

#endif // HARKIO_HARKCONFIG_NODEPARSERS_HPP_

