#include <harkio2/hark/HarkFileBinaryReader.hpp>

#include <boost/static_assert.hpp>
#include <harkio2/hark/HarkFile.hpp>

// load Helper implementation.
#include "HarkFileBinaryReader/Helper.cpp"

using namespace std;
using namespace boost;

BOOST_STATIC_ASSERT(sizeof(float) == 4);

namespace harkio2
{
namespace hark
{

HarkFileBinaryReader::HarkFileBinaryReader(std::istream& in)
    : in(in)
{ }

boost::shared_ptr<harkio2::hark::HarkFile> HarkFileBinaryReader::Read() throw (harkio2::AnalyzeError)
{
    const char* funcName = "HarkFileBinaryReader::Read";
    streampos prevPos = 0; // variable for rewinding position.

    // calculate file size
    in.seekg(0, ios::end);
    streampos end = in.tellg();
    in.seekg(0, ios::beg);
    streampos beg = in.tellg();
    size_t fileSize = end - beg;

    Helper helper(*this);

    char tag[4];
    helper.Read(tag, 4);
    uint32_t size;
    helper.Read(size);

    if (strncmp(tag, "HARK", 4) != 0)
    {
        throw harkio2::AnalyzeError((format("%s: this is not HARK binary file") % funcName).str());
    }
    else if (size != fileSize - 8)
    {
        throw harkio2::AnalyzeError((format("%s: invalid size property") % funcName).str());
    }

    // get tag "HGTF"
    prevPos = in.tellg();
    helper.Read(tag, 4);
    in.seekg(prevPos, ios::beg);

    if (strncmp(tag, "HGTF", 4) == 0)
    {
        return helper.ReadHGTF();
    }
    else if (strncmp(tag, "HGFB", 4) == 0)
    {
        return helper.ReadHGFB();
    }
    else
    {
        throw harkio2::AnalyzeError((format("%s: unknown chunk id %s") % funcName % tag).str());
    }
}

}
}

