#include <harkio2/harkconfig/HarkConfigFileTextReader.hpp>

#include <cctype>
#include <boost/scoped_ptr.hpp>

#include <harkio2/AnalyzeError.hpp>
#include <harkio2/harkconfig/HarkConfigFile.hpp>
#include <harkio2/harkconfig/HarkConfigFileTextHelper.hpp>
#include <harkio2/harkconfig/ImpulseResponseListFileTextHelper.hpp>

using namespace std;
using namespace boost;

namespace harkio2
{
namespace harkconfig
{

HarkConfigFileTextReader::HarkConfigFileTextReader(std::istream& in) :
    in(in)
{
}

boost::shared_ptr<HarkConfigFile> HarkConfigFileTextReader::Read()
    throw (harkio2::AnalyzeError)
{
    string line;
    scoped_ptr<HarkConfigFileTextHelper> helper;
    unsigned int readLines = 0, totalLines = 0;

    if (GetValidLine(in, line, readLines))
    {
        totalLines = readLines;
        if (line == "ImpulseResponse-ListFile")
        {
            helper.reset(new ImpulseResponseListFileTextHelper());
        }
        else
        {
            throw AnalyzeError("unknown type identifier - " + line, totalLines);
        }
    }

    if (helper)
    {
        while (GetValidLine(in, line, readLines))
        {
            totalLines += readLines;
            helper->AnalyzeLine(line, totalLines);
        }
        return helper->GetData();
    }

    return boost::shared_ptr<HarkConfigFile>();
}

std::istream& HarkConfigFileTextReader::GetValidLine(std::istream& in, std::string& line)
{
    string rawline;
    while (getline(in, rawline))
    {
        string noSpaceLine = Shrink(EliminateComment(rawline));
        if (noSpaceLine.length() > 0)
        {
            line.swap(noSpaceLine);
            break;
        }
    }
    return in;
}

std::istream& HarkConfigFileTextReader::GetValidLine(std::istream& in, std::string& line, unsigned int& numReadLine)
{
    string rawline;
    numReadLine = 0;
    while (getline(in, rawline))
    {
        numReadLine++;
        string noSpaceLine = Shrink(EliminateComment(rawline));
        if (noSpaceLine.length() > 0)
        {
            line.swap(noSpaceLine);
            break;
        }
    }
    return in;
}

std::string HarkConfigFileTextReader::EliminateComment(const std::string& line)
{
    string::size_type commentStart = line.find('#');

    if (commentStart == string::npos)
    {
        return line;
    }

    return line.substr(0, commentStart);
}

std::string HarkConfigFileTextReader::Shrink(const std::string& line)
{
    string::size_type begin = 0, end = line.length();

    while (begin < end && isspace(line[begin]))
    {
        begin++;
    }

    while (begin < end && isspace(line[end - 1]))
    {
        end--;
    }

    return line.substr(begin, end - begin);
}

}
}

